package com.ceylonlab.is_device_secure

import android.app.KeyguardManager
import android.content.Context
import android.os.Build
import android.os.Build.VERSION
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.flutter.plugin.common.BinaryMessenger;

/** IsDeviceSecurePlugin */
public class IsDeviceSecurePlugin : MethodCallHandler, FlutterPlugin {
    private var applicationContext: Context? = null
    private var methodChannel: MethodChannel? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        onAttachedToEngine(flutterPluginBinding.getApplicationContext(), flutterPluginBinding.binaryMessenger)
    }

    private fun onAttachedToEngine(applicationContext: Context,messenger: BinaryMessenger) {
        this.applicationContext = applicationContext
        methodChannel = MethodChannel(messenger, "is_device_secure")
        methodChannel?.setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        applicationContext = null
        methodChannel?.setMethodCallHandler(null)
        methodChannel = null
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "getPlatformVersion") {
            result.success("Android ${VERSION.RELEASE}")
        } else if (call.method == "isSecure") {
            result.success(isDeviceSecured());
        } else {
            result.notImplemented()
        }
    }

    private fun isDeviceSecured(): Boolean {
        val keyguardManager = applicationContext?.getSystemService(Context.KEYGUARD_SERVICE) as KeyguardManager //api 16+
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            keyguardManager.isDeviceSecure
        } else keyguardManager.isKeyguardSecure
    }

    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val instance = IsDeviceSecurePlugin()
            instance.onAttachedToEngine(registrar.context(), registrar.messenger())
        }
    }
}